@@ -1,13 +1,16 @@
 import time
 import logging
 import functools
+import json
 
 import bizerror
 from fastutils import rsautils
-from fastutils.funcutils import call_with_inject
-from fastutils.typingutils import Number
-from fastutils.typingutils import smart_cast
+from fastutils import strutils
+from fastutils import funcutils
+from fastutils import typingutils
+
 from django.http import JsonResponse
+from django.conf import settings
 
 from .utils import SimpleJsonEncoder
 from .utils import get_request_data
@@ -24,10 +27,12 @@ def setup_result_packer(packer):
 class View(object):
     """Process class of apiview.
     """
-    def __init__(self, request, **kwargs):
+    def __init__(self, request, defaults, **kwargs):
         self.request = request
         self.kwargs = kwargs
-        self.data = get_request_data(self.request, self.kwargs)
+        self.data = {}
+        self.data.update(defaults)
+        self.data.update(get_request_data(self.request, self.kwargs))
         self.data["_django_apiview_view_instance"] = self
 
     def process(self, func):
@@ -37,7 +42,7 @@ class View(object):
             if not str(error).endswith("got an unexpected keyword argument '_django_apiview_view_instance'"):
                 raise error
             else:
-                return call_with_inject(func, self.data)
+                return funcutils.call_with_inject(func, self.data)
 
 class Apiview(object):
     def __init__(self, packer):
@@ -47,7 +52,8 @@ class Apiview(object):
         """Turn the view function into apiview function. Must use as the first decorator.
         """
         def wrapper(request, **kwargs):
-            view = View(request,**kwargs)
+            defaults = funcutils.get_default_values(func)
+            view = View(request, defaults, **kwargs)
             package = {}
             try:
                 result = view.process(func)
@@ -92,7 +98,7 @@ def choices(field, choices, annotation=None, allow_none=False):
                 values = choices
             value = view.data.get(field, None)
             if annotation:
-                value = smart_cast(annotation, value)
+                value = typingutils.smart_cast(annotation, value)
             if (allow_none and value is None) or (value in choices):
                 return view.process(func)
             else:
@@ -100,7 +106,7 @@ def choices(field, choices, annotation=None, allow_none=False):
         return functools.wraps(func)(wrapper)
     return wrapper_outer
 
-def between(field, min, max, include_min=True, include_max=True, annotation=Number, allow_none=False):
+def between(field, min, max, include_min=True, include_max=True, annotation=typingutils.Number, allow_none=False):
     """Make sure field's numeric value is in range of [min, max].
     """
     def wrapper_outer(func):
@@ -117,7 +123,7 @@ def between(field, min, max, include_min=True, include_max=True, annotation=Numb
             else:
                 max_value = max
             value = view.data.get(field, None)
-            value = smart_cast(Number, value)
+            value = typingutils.smart_cast(typingutils.Number, value)
             if (allow_none and value is None) or ((include_min and min_value <= value or min_value < value) and (include_max and max_value >= value or max_value > value)):
                 return view.process(func)
             else:
@@ -159,3 +165,71 @@ def meta_variable(variable_name, meta_name):
             return view.process(func)
         return functools.wraps(func)(wrapper)
     return wrapper_outer
+
+
+def cache(key, expire=None, cache_name="default", get_from_cache=None, set_to_cache=True, disable_get_from_cache_header=getattr(settings, "DJANGO_APIVIEW_DISABLE_CACHE_HEADER_NAME", "HTTP_DISABLE_CACHE"), ignore_cache_errors=True):
+    # Import here, so that we don't need django-redis by default.
+    # Only if you use cache, then you need pip install django-redis
+    from django_redis import get_redis_connection
+
+    expire = expire or getattr(settings, "DJANGO_APIVIEW_DEFAULT_CACHE_EXPIRE", None)
+    key_template = key
+    def wrapper_outer(func):
+        def wrapper(_django_apiview_view_instance):
+            view = _django_apiview_view_instance
+            key = strutils.format_with_mapping(
+                key_template,
+                funcutils.chain(strutils.none_to_empty_string, strutils.strip_string),
+                **view.data,
+                )
+
+            if get_from_cache == False:
+                get_from_cache_final = False
+            else:
+                if view.request.META.get(disable_get_from_cache_header, "false") == "true":
+                    get_from_cache_final = False
+                else:
+                    get_from_cache_final = True
+
+            set_to_cache_final = set_to_cache
+            use_cache = get_from_cache_final or set_to_cache_final
+
+            cache = None
+            if use_cache:
+                try:
+                    cache = get_redis_connection(cache_name)
+                except Exception as error:
+                    logger.error("get redis connection failed @cache, error_message={0}".format(str(error)))
+                    if ignore_cache_errors:
+                        cache = None
+                    else:
+                        raise
+    
+            if cache and get_from_cache:
+                try:
+                    result_text = cache.get(key)
+                    if not result_text is None:
+                        return json.loads(result_text)
+                except Exception as error:
+                    logger.error("query redis failed @cache, error_message={0}".format(str(error)))
+                    if not ignore_cache_errors:
+                        raise
+
+            result = view.process(func)
+
+            if cache and set_to_cache:
+                try:
+                    result_text = json.dumps(result, cls=SimpleJsonEncoder, allow_nan=True, sort_keys=True)
+                    if expire:
+                        cache.set(key, result_text,  keepttl=expire)
+                    else:
+                        cache.set(key, result_text)
+                except Exception as error:
+                    logger.error("write redis failed @cache, error_message={0}".format(str(error)))
+                    if not ignore_cache_errors:
+                        raise
+
+            return result
+        return functools.wraps(func)(wrapper)
+    return wrapper_outer
+
